package index.build;

import java.io.IOException;
import java.util.Set;

import org.apache.lucene.analysis.WhitespaceAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.apache.lucene.document.Field.TermVector;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.LockObtainFailedException;

import parse.LuceneConfiguration;
import source.file.SourceReader;
import value.Row;

public class IndexBuilder {

    private LuceneConfiguration luceneConfiguration;

    private TokenCreator tokenCreator;

    private SourceReader<Row> sourceReader;

    public void setSourceReader(SourceReader<Row> sourceReader) {
        this.sourceReader = sourceReader;
    }

    public IndexBuilder(LuceneConfiguration luceneConfiguration) {
        this.luceneConfiguration = luceneConfiguration;
    }

    public void buildIndex() {
        try {
            IndexWriter indexWriter = getWriter();
            tokenCreator = new TokenCreator(luceneConfiguration);
            addDocuments(indexWriter);
            indexWriter.optimize();
            indexWriter.close();
        } catch (CorruptIndexException e) {
            System.out.println("***********Corrupted Index*********");
        } catch (LockObtainFailedException e) {
            System.out.println("***********Lock not found**********");
        } catch (IOException e) {
            System.out.println("***********IO Exception************");
        }
    }

    private void addDocuments(IndexWriter indexWriter) throws CorruptIndexException, IOException {
        sourceReader.setSeparator(luceneConfiguration.getSeparator());
        while (sourceReader.hasNext()) {
            Row row = sourceReader.next();
            indexWriter.addDocument(createDocument(row));
        }
    }

    private Document createDocument(Row row) {
        Document document = new Document();
        Set<String> fields = luceneConfiguration.getPerFieldInfo().keySet();
        for (String fieldName : fields) {
            boolean isStore = luceneConfiguration.getPerFieldInfo().get(fieldName).isStore();
            Store store = getStore(isStore);
            Field field = new Field(fieldName, row.get(fieldName), store, Index.ANALYZED, TermVector.NO);
            document.add(field);
        }
        return document;
    }

    private Store getStore(boolean isStore) {
        Store store = null;
        if (isStore) {
            store = Store.YES;
        } else {
            store = Store.NO;
        }
        return store;
    }

    private IndexWriter getWriter() throws CorruptIndexException, LockObtainFailedException, IOException {
        IndexWriter indexWriter = new IndexWriter(FSDirectory.open(luceneConfiguration.getIndexLocation()), new WhitespaceAnalyzer(),
                luceneConfiguration.isOverWrite(), IndexWriter.MaxFieldLength.UNLIMITED);
        return indexWriter;
    }
}